const sample = [
    {
      language: 'Rust',
      value: 78.9,
      color: '#000000'
    },
    {
      language: 'Kotlin',
      value: 75.1,
      color: '#00a2ee'
    },
    {
      language: 'Python',
      value: 68.0,
      color: '#fbcb39'
    },
    {
      language: 'TypeScript',
      value: 67.0,
      color: '#007bc8'
    },
    {
      language: 'Go',
      value: 65.6,
      color: '#65cedb'
    },
    {
      language: 'Swift',
      value: 65.1,
      color: '#ff6e52'
    },
    {
      language: 'JavaScript',
      value: 61.9,
      color: '#f9de3f'
    },
    {
      language: 'C#',
      value: 60.4,
      color: '#5d2f8e'
    },
    {
      language: 'F#',
      value: 59.6,
      color: '#008fc9'
    },
    {
      language: 'Clojure',
      value: 59.6,
      color: '#507dca'
    }
  ];
  
  const svg = d3.select('svg');
  //const svgContainer = d3.select('#container');
  const margin = 60;
  const width = 1000 - 2 * margin;
  const height = 600 - 2 * margin;
  
// Translation mathématique de l'element: on le fait glisser par 60 pixels
	const chart = svg.append('g')
    .attr('transform', `translate(${margin}, ${margin})`);
  
  const yScale = d3.scaleLinear()
    .range([height, 0])
    .domain([0, 100]);
  
  chart.append('g')
    .call(d3.axisLeft(yScale));
  
  const xScale = d3.scaleBand()
    .range([0, width])
    .domain(sample.map((s) => s.language))
    .padding(0.2);

// Translation mathématique de l'element: dessiner l'axe des y en bas
  chart.append('g')
      .attr('transform', `translate(0, ${height})`)
      .call(d3.axisBottom(xScale));

// link the data to the DOM element
  const barGroups = chart.selectAll()
  .data(sample)
  .enter()
  .append('g');

// draw rect linked to every data element
  barGroups
    .append('rect')
    .attr('class', 'bar')
    .attr('x', (g) => xScale(g.language))
    .attr('y', (g) => yScale(g.value))
    .attr('height', (g) => height - yScale(g.value))
    .attr('width', xScale.bandwidth())
		//mouse enter
		.on('mouseenter', function (actual, i) {
		//opacity 
    d3.select(this)
      .transition()
      .duration(300)
      .attr('opacity', 0.6);

    const y = yScale(actual.value);

    line = chart.append('line')
      .attr('id', 'limit')
      .attr('x1', 0)
      .attr('y1', y)
      .attr('x2', width)
      .attr('y2', y);

    barGroups.append('text')
      .attr('class', 'divergence')
      .attr('x', (a) => xScale(a.language) + xScale.bandwidth() / 2)
      .attr('y', (a) => yScale(a.value) + 30)
      .attr('fill', 'white')
      .attr('text-anchor', 'middle')
      .text((a, idx) => {
        const divergence = (a.value - actual.value).toFixed(1)

        let text = ''
        if (divergence > 0) text += '+'
        text += `${divergence}%`

        return idx !== i ? text : ''; 
    	})
		})
	//mouse leave
		.on('mouseleave', function () {
    d3.selectAll('.value')
      .attr('opacity', 1)

    d3.select(this)
      .transition()
      .duration(300)
      .attr('opacity', 1)
      .attr('x', (a) => xScale(a.language))
      .attr('width', xScale.bandwidth())

    chart.selectAll('#limit').remove()
    chart.selectAll('.divergence').remove()
  });
		

//draw gridlines
  chart.append('g')
      .attr('class', 'grid')
      .call(d3.axisLeft()
          .scale(yScale)
          .tickSize(-width, 0, 0)
          .tickFormat(''));

//Titre à droite
  svg.append('text')
      .attr('x', -(height / 2) - margin)
      .attr('y', margin / 2.4)
      .attr('transform', 'rotate(-90)')
      .attr('text-anchor', 'middle')
      .text('Love meter (%)');

//Titre en haut
  svg.append('text')
      .attr('x', width / 2 + margin)
      .attr('y', 40)
      .attr('text-anchor', 'middle')
      .text('Most loved programming languages in 2018